@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003 Red Hat, Inc.
+ * Copyright (C) 2003,2004 Red Hat, Inc.
  *
  * This is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Library General Public License as published by
@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: vtefc.c,v 1.9 2003/05/20 06:14:27 nalin Exp $"
+#ident "$Id: vtefc.c,v 1.15 2004/04/20 05:16:56 nalin Exp $"
 
 #include "../config.h"
 
@@ -46,7 +46,7 @@ _vte_fc_weight_from_pango_weight(int weight)
 }
 
 static int
-_vte_fc_slant_from_pango_style(int style)
+_vte_fc_slant_from_pango_style(PangoStyle style)
 {
 	switch (style) {
 	case PANGO_STYLE_NORMAL:
@@ -62,16 +62,57 @@ _vte_fc_slant_from_pango_style(int style)
 	return FC_SLANT_ROMAN;
 }
 
+static int
+_vte_fc_width_from_pango_stretch(PangoStretch stretch)
+{
+	switch (stretch) {
+	case PANGO_STRETCH_ULTRA_CONDENSED:
+		return 60;
+		break;
+	case PANGO_STRETCH_EXTRA_CONDENSED:
+		return 70;
+		break;
+	case PANGO_STRETCH_CONDENSED:
+		return 80;
+		break;
+	case PANGO_STRETCH_SEMI_CONDENSED:
+		return 90;
+		break;
+	case PANGO_STRETCH_NORMAL:
+		return 100;
+		break;
+	case PANGO_STRETCH_SEMI_EXPANDED:
+		return 105;
+		break;
+	case PANGO_STRETCH_EXPANDED:
+		return 120;
+		break;
+	case PANGO_STRETCH_EXTRA_EXPANDED:
+		return 150;
+		break;
+	case PANGO_STRETCH_ULTRA_EXPANDED:
+		return 200;
+		break;
+	}
+	return 100;
+}
+
 static void
-_vte_fc_transcribe_from_pango_font_description(FcPattern *pattern,
+_vte_fc_transcribe_from_pango_font_description(GtkWidget *widget,
+					       FcPattern *pattern,
 				       const PangoFontDescription *font_desc)
 {
+#if GTK_CHECK_VERSION(2,2,0)
+	GdkScreen *screen;
+#endif
 	const char *family = "monospace";
 	PangoLanguage *language;
 	double size = 10.0;
 	int pango_mask;
 	PangoContext *context;
-	int weight, style;
+	PangoWeight weight;
+	PangoStyle style;
+	PangoStretch stretch;
 
 	if (font_desc == NULL) {
 		return;
@@ -94,7 +135,12 @@ _vte_fc_transcribe_from_pango_font_description(FcPattern *pattern,
 
 	/* Set the language for the pattern. */
 #if GTK_CHECK_VERSION(2,2,0)
-	context = gdk_pango_context_get_for_screen(gdk_screen_get_default());
+	if (gtk_widget_has_screen(widget)) {
+		screen = gtk_widget_get_screen(widget);
+	} else {
+		screen = gdk_display_get_default_screen(gtk_widget_get_display(widget));
+	}
+	context = gdk_pango_context_get_for_screen(screen);
 #else
 	context = gdk_pango_context_get();
 #endif
@@ -112,15 +158,39 @@ _vte_fc_transcribe_from_pango_font_description(FcPattern *pattern,
 				    _vte_fc_weight_from_pango_weight(weight));
 	}
 
+	if (pango_mask & PANGO_FONT_MASK_STRETCH) {
+		stretch = pango_font_description_get_stretch(font_desc);
+		FcPatternAddInteger(pattern, FC_WIDTH,
+				    _vte_fc_width_from_pango_stretch(stretch));
+	}
+
 	if (pango_mask & PANGO_FONT_MASK_STYLE) {
 		style = pango_font_description_get_style(font_desc);
 		FcPatternAddInteger(pattern, FC_SLANT,
 				    _vte_fc_slant_from_pango_style(style));
 	}
+
+	g_object_unref(G_OBJECT(context));
 }
 
 static void
-_vte_fc_defaults_from_gtk(FcPattern *pattern)
+_vte_fc_set_antialias(FcPattern *pattern, VteTerminalAntiAlias antialias)
+{
+	FcBool aa;
+	if (antialias != VTE_ANTI_ALIAS_USE_DEFAULT) {
+		if (FcPatternGetBool(pattern, FC_ANTIALIAS, 0,
+				     &aa) != FcResultNoMatch) {
+			FcPatternDel(pattern, FC_ANTIALIAS);
+		}
+		aa = (antialias == VTE_ANTI_ALIAS_FORCE_ENABLE) ?
+		     FcTrue : FcFalse;
+		FcPatternAddBool(pattern, FC_ANTIALIAS, aa);
+	}
+}
+
+static void
+_vte_fc_defaults_from_gtk(GtkWidget *widget, FcPattern *pattern,
+			  VteTerminalAntiAlias explicit_antialias)
 {
 	GtkSettings *settings;
 #if GTK_CHECK_VERSION(2,2,0)
@@ -132,7 +202,11 @@ _vte_fc_defaults_from_gtk(FcPattern *pattern)
 
 	/* Add any defaults configured for GTK+. */
 #if GTK_CHECK_VERSION(2,2,0)
-	screen = gdk_screen_get_default();
+	if (gtk_widget_has_screen(widget)) {
+		screen = gtk_widget_get_screen(widget);
+	} else {
+		screen = gdk_display_get_default_screen(gtk_widget_get_display(widget));
+	}
 	settings = gtk_settings_get_for_screen(screen);
 #else
 	settings = gtk_settings_get_default();
@@ -161,6 +235,7 @@ _vte_fc_defaults_from_gtk(FcPattern *pattern)
 		FcPatternDel(pattern, FC_ANTIALIAS);
 		FcPatternAddBool(pattern, FC_ANTIALIAS, antialias > 0);
 	}
+	_vte_fc_set_antialias(pattern, explicit_antialias);
 
 	/* Pick up the configured DPI setting. */
 	if (dpi >= 0) {
@@ -244,7 +319,8 @@ _vte_fc_defaults_from_gtk(FcPattern *pattern)
 }
 
 static void
-_vte_fc_defaults_from_rdb(FcPattern *pattern)
+_vte_fc_defaults_from_rdb(GtkWidget *widget, FcPattern *pattern,
+			  VteTerminalAntiAlias explicit_antialias)
 {
 	FcBool fcb;
 	double fcd;
@@ -253,34 +329,46 @@ _vte_fc_defaults_from_rdb(FcPattern *pattern)
 	const char *rgba = NULL, *hintstyle = NULL;
 
 	/* Read the settings. */
-	hintstyle = _vte_rdb_get_hintstyle();
-	rgba = _vte_rdb_get_rgba();
+	hintstyle = _vte_rdb_get_hintstyle(widget);
+	rgba = _vte_rdb_get_rgba(widget);
 
 	/* Pick up the antialiasing setting. */
 	if (FcPatternGetBool(pattern, FC_ANTIALIAS, 0,
 			     &fcb) == FcResultNoMatch) {
-		antialias = _vte_rdb_get_antialias();
+		antialias = _vte_rdb_get_antialias(widget);
 		FcPatternAddBool(pattern, FC_ANTIALIAS, antialias);
 	}
+	if (explicit_antialias != VTE_ANTI_ALIAS_USE_DEFAULT) {
+		FcBool aa;
+		if (FcPatternGetBool(pattern, FC_ANTIALIAS, 0,
+				     &aa) != FcResultNoMatch) {
+			FcPatternDel(pattern, FC_ANTIALIAS);
+		}
+		aa = (explicit_antialias == VTE_ANTI_ALIAS_FORCE_ENABLE) ?
+		     FcTrue : FcFalse;
+		FcPatternAddBool(pattern, FC_ANTIALIAS, aa);
+	}
 
 	/* Pick up the hinting setting. */
 	if (FcPatternGetBool(pattern, FC_HINTING, 0,
 			     &fcb) == FcResultNoMatch) {
-		hinting = _vte_rdb_get_hinting();
+		hinting = _vte_rdb_get_hinting(widget);
 		FcPatternAddBool(pattern, FC_HINTING, hinting);
 	}
 
 	/* Pick up the configured DPI setting. */
 	if (FcPatternGetDouble(pattern, FC_DPI, 0,
 			       &fcd) == FcResultNoMatch) {
-		dpi = _vte_rdb_get_dpi();
-		FcPatternAddDouble(pattern, FC_DPI, dpi);
+		dpi = _vte_rdb_get_dpi(widget);
+		if (dpi >= 0) {
+			FcPatternAddDouble(pattern, FC_DPI, dpi);
+		}
 	}
 
 	/* Pick up the configured subpixel rendering setting. */
 	if (FcPatternGetInteger(pattern, FC_RGBA, 0,
 				&fci) == FcResultNoMatch) {
-		rgba = _vte_rdb_get_rgba();
+		rgba = _vte_rdb_get_rgba(widget);
 		if (g_ascii_strcasecmp(rgba, "none") == 0) {
 			FcPatternAddInteger(pattern, FC_RGBA, FC_RGBA_NONE);
 		} else
@@ -302,7 +390,7 @@ _vte_fc_defaults_from_rdb(FcPattern *pattern)
 	/* Pick up the default hinting style. */
 	if (FcPatternGetInteger(pattern, FC_HINT_STYLE, 0,
 				&fci) == FcResultNoMatch) {
-		hintstyle = _vte_rdb_get_hintstyle();
+		hintstyle = _vte_rdb_get_hintstyle(widget);
 		if (g_ascii_strcasecmp(hintstyle, "hintnone") == 0) {
 			FcPatternAddInteger(pattern, FC_HINT_STYLE,
 					    FC_HINT_NONE);
@@ -326,13 +414,15 @@ _vte_fc_defaults_from_rdb(FcPattern *pattern)
 /* Create a sorted set of fontconfig patterns from a Pango font description
  * and append them to the array. */
 gboolean
-_vte_fc_patterns_from_pango_font_desc(const PangoFontDescription *font_desc,
+_vte_fc_patterns_from_pango_font_desc(GtkWidget *widget,
+				      const PangoFontDescription *font_desc,
+				      VteTerminalAntiAlias antialias,
 				      GArray *pattern_array,
 				      _vte_fc_defaults_cb defaults_cb,
 				      gpointer defaults_data)
 
 {
-	FcPattern *pattern, *match, *tmp;
+	FcPattern *pattern, *match, *tmp, *save;
 	FcFontSet *fontset;
 	FcResult result;
 	gboolean ret = FALSE;
@@ -344,16 +434,20 @@ _vte_fc_patterns_from_pango_font_desc(const PangoFontDescription *font_desc,
 	pattern = FcPatternCreate();
 
 	/* Transcribe what we can get from the Pango font description. */
-	_vte_fc_transcribe_from_pango_font_description(pattern, font_desc);
+	_vte_fc_transcribe_from_pango_font_description(widget, pattern,
+						       font_desc);
 
 	/* Add any defaults specified in the configuration. */
 	FcConfigSubstitute(NULL, pattern, FcMatchPattern);
 
 	/* Add any defaults configured for GTK+. */
-	_vte_fc_defaults_from_gtk(pattern);
+	_vte_fc_defaults_from_gtk(widget, pattern, antialias);
 
 	/* Add defaults configured via the resource database. */
-	_vte_fc_defaults_from_rdb(pattern);
+	_vte_fc_defaults_from_rdb(widget, pattern, antialias);
+
+	/* Add any hard-coded default for antialiasing. */
+	_vte_fc_set_antialias(pattern, antialias);
 
 	/* Add any defaults which are hard-coded in fontconfig. */
 	FcDefaultSubstitute(pattern);
@@ -371,8 +465,11 @@ _vte_fc_patterns_from_pango_font_desc(const PangoFontDescription *font_desc,
 			tmp = FcFontRenderPrepare(NULL,
 						  pattern,
 						  fontset->fonts[i]);
-			_vte_fc_defaults_from_gtk(tmp);
-			g_array_append_val(pattern_array, tmp);
+			_vte_fc_defaults_from_gtk(widget, tmp, antialias);
+			_vte_fc_set_antialias(tmp, antialias);
+			save = FcPatternDuplicate(tmp);
+			FcPatternDestroy(tmp);
+			g_array_append_val(pattern_array, save);
 		}
 		FcFontSetDestroy(fontset);
 		ret = TRUE;
@@ -383,8 +480,11 @@ _vte_fc_patterns_from_pango_font_desc(const PangoFontDescription *font_desc,
 		match = FcFontMatch(NULL, pattern, &result);
 		if (result == FcResultMatch) {
 			tmp = FcPatternDuplicate(match);
-			_vte_fc_defaults_from_gtk(tmp);
-			g_array_append_val(pattern_array, tmp);
+			_vte_fc_defaults_from_gtk(widget, tmp, antialias);
+			_vte_fc_set_antialias(tmp, antialias);
+			save = FcPatternDuplicate(tmp);
+			FcPatternDestroy(tmp);
+			g_array_append_val(pattern_array, save);
 			ret = TRUE;
 		} else {
 			ret = FALSE;
