@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: vtegl.c,v 1.14 2003/06/16 21:15:10 nalin Exp $"
+#ident "$Id: vtegl.c,v 1.15 2004/04/20 05:16:56 nalin Exp $"
 
 #include "../config.h"
 
@@ -384,7 +384,8 @@ _vte_gl_fcpattern_disable_rgba(FcPattern *pattern, gpointer data)
 
 static void
 _vte_gl_set_text_font(struct _vte_draw *draw,
-		      const PangoFontDescription *fontdesc)
+		     const PangoFontDescription *fontdesc,
+		     VteTerminalAntiAlias antialias)
 {
 	struct _vte_gl_data *data;
 
@@ -397,6 +398,7 @@ _vte_gl_set_text_font(struct _vte_draw *draw,
 
 	_vte_glyph_cache_set_font_description(draw->widget,
 					      NULL, data->cache, fontdesc,
+					      antialias,
 					      _vte_gl_fcpattern_disable_rgba,
 					      NULL);
 }
