@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: vtepango.c,v 1.9 2003/05/06 07:38:39 nalin Exp $"
+#ident "$Id: vtepango.c,v 1.16 2004/04/20 05:16:56 nalin Exp $"
 
 #include "../config.h"
 
@@ -37,6 +37,7 @@ struct _vte_pango_data
 	GdkPixmap *pixmap;
 	gint pixmapw, pixmaph;
 	gint scrollx, scrolly;
+	PangoContext *ctx;
 	PangoFontDescription *font;
 	PangoLayout *layout;
 	GdkGC *gc;
@@ -89,6 +90,10 @@ _vte_pango_destroy(struct _vte_draw *draw)
 		g_object_unref(G_OBJECT(data->layout));
 		data->layout = NULL;
 	}
+	if (PANGO_IS_CONTEXT(data->ctx)) {
+		g_object_unref(G_OBJECT(data->ctx));
+		data->ctx = NULL;
+	}
 	if (GDK_IS_GC(data->gc)) {
 		g_object_unref(G_OBJECT(data->gc));
 		data->gc = NULL;
@@ -127,11 +132,15 @@ _vte_pango_start(struct _vte_draw *draw)
 #else
 	ctx = gdk_pango_context_get();
 #endif
+	if (PANGO_IS_CONTEXT(data->ctx)) {
+		g_object_unref(G_OBJECT(data->ctx));
+	}
+	data->ctx = ctx;
 
 	if (PANGO_IS_LAYOUT(data->layout)) {
 		g_object_unref(G_OBJECT(data->layout));
 	}
-	data->layout = pango_layout_new(ctx);
+	data->layout = pango_layout_new(data->ctx);
 	if (data->font) {
 		pango_layout_set_font_description(data->layout, data->font);
 	}
@@ -151,15 +160,20 @@ _vte_pango_end(struct _vte_draw *draw)
 	struct _vte_pango_data *data;
 	data = (struct _vte_pango_data*) draw->impl_data;
 
+	if (GDK_IS_GC(data->gc)) {
+		g_object_unref(G_OBJECT(data->gc));
+	}
+	data->gc = NULL;
+
 	if (PANGO_IS_LAYOUT(data->layout)) {
 		g_object_unref(G_OBJECT(data->layout));
 	}
 	data->layout = NULL;
 
-	if (GDK_IS_GC(data->gc)) {
-		g_object_unref(G_OBJECT(data->gc));
+	if (PANGO_IS_CONTEXT(data->ctx)) {
+		g_object_unref(G_OBJECT(data->ctx));
 	}
-	data->gc = NULL;
+	data->ctx = NULL;
 }
 
 static void
@@ -172,19 +186,19 @@ _vte_pango_set_background_color(struct _vte_draw *draw, GdkColor *color)
 
 static void
 _vte_pango_set_background_image(struct _vte_draw *draw,
-			        enum VteBgSourceType type,
-			        GdkPixbuf *pixbuf,
-			        const char *file,
-			        const GdkColor *color,
-			        double saturation)
+				enum VteBgSourceType type,
+				GdkPixbuf *pixbuf,
+				const char *file,
+				const GdkColor *color,
+				double saturation)
 {
 	GdkPixmap *pixmap;
 	struct _vte_pango_data *data;
 
 	data = (struct _vte_pango_data*) draw->impl_data;
 	pixmap = vte_bg_get_pixmap(vte_bg_get(), type, pixbuf, file,
-				   color, saturation, 
-				   _vte_draw_get_colormap(draw));
+				   color, saturation,
+				   _vte_draw_get_colormap(draw, TRUE));
 	if (GDK_IS_PIXMAP(data->pixmap)) {
 		g_object_unref(G_OBJECT(data->pixmap));
 	}
@@ -244,7 +258,8 @@ _vte_pango_clear(struct _vte_draw *draw,
 
 static void
 _vte_pango_set_text_font(struct _vte_draw *draw,
-			 const PangoFontDescription *fontdesc)
+			 const PangoFontDescription *fontdesc,
+			 VteTerminalAntiAlias antialias)
 {
 #if GTK_CHECK_VERSION(2,2,0)
 	GdkScreen *screen;
@@ -265,7 +280,7 @@ _vte_pango_set_text_font(struct _vte_draw *draw,
 	if (gtk_widget_has_screen(draw->widget)) {
 		screen = gtk_widget_get_screen(draw->widget);
 	} else {
-		screen = gdk_screen_get_default();
+		screen = gdk_display_get_default_screen(gtk_widget_get_display(draw->widget));
 	}
 	ctx = gdk_pango_context_get_for_screen(screen);
 #else
@@ -325,6 +340,7 @@ _vte_pango_set_text_font(struct _vte_draw *draw,
 #endif
 
 	g_object_unref(G_OBJECT(layout));
+	g_object_unref(G_OBJECT(ctx));
 }
 
 static int
@@ -345,6 +361,12 @@ _vte_pango_get_text_ascent(struct _vte_draw *draw)
 	return draw->ascent;
 }
 
+static int
+_vte_pango_get_char_width(struct _vte_draw *draw, gunichar c, int columns)
+{
+	return _vte_pango_get_text_width(draw) * columns;
+}
+
 static gboolean
 _vte_pango_get_using_fontconfig(struct _vte_draw *draw)
 {
@@ -381,6 +403,15 @@ _vte_pango_draw_text(struct _vte_draw *draw,
 	}
 }
 
+static gboolean
+_vte_pango_draw_char(struct _vte_draw *draw,
+		     struct _vte_draw_text_request *request,
+		     GdkColor *color, guchar alpha)
+{
+	_vte_pango_draw_text(draw, request, 1, color, alpha);
+	return TRUE;
+}
+
 static void
 _vte_pango_draw_rectangle(struct _vte_draw *draw,
 			  gint x, gint y, gint width, gint height,
@@ -437,13 +468,16 @@ struct _vte_draw_impl _vte_draw_pango = {
 	_vte_pango_end,
 	_vte_pango_set_background_color,
 	_vte_pango_set_background_image,
+	FALSE,
 	_vte_pango_clear,
 	_vte_pango_set_text_font,
 	_vte_pango_get_text_width,
 	_vte_pango_get_text_height,
 	_vte_pango_get_text_ascent,
+	_vte_pango_get_char_width,
 	_vte_pango_get_using_fontconfig,
 	_vte_pango_draw_text,
+	_vte_pango_draw_char,
 	_vte_pango_draw_rectangle,
 	_vte_pango_fill_rectangle,
 	_vte_pango_set_scroll,
