@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: vtepangox.c,v 1.13 2003/06/16 21:15:10 nalin Exp $"
+#ident "$Id: vtepangox.c,v 1.14 2004/04/20 05:16:56 nalin Exp $"
 
 #include "../config.h"
 
@@ -271,7 +271,8 @@ _vte_pango_x_clear(struct _vte_draw *draw,
 
 static void
 _vte_pango_x_set_text_font(struct _vte_draw *draw,
-			   const PangoFontDescription *fontdesc)
+			   const PangoFontDescription *fontdesc,
+			   VteTerminalAntiAlias antialias)
 {
 	PangoContext *ctx;
 	Display *display;
